@@ -1,9 +1,10 @@
 import hazelcast.util as util
-from hazelcast.exception import HazelcastSerializationError
+from hazelcast.errors import HazelcastSerializationError
 from hazelcast.serialization import INT_SIZE_IN_BYTES, NULL_ARRAY_LENGTH
 from hazelcast.serialization.api import PortableWriter
 from hazelcast.serialization.output import EmptyObjectDataOutput
 from hazelcast.serialization.portable.classdef import FieldType, ClassDefinitionBuilder
+from hazelcast.six.moves import range
 
 
 class DefaultPortableWriter(PortableWriter):
@@ -56,9 +57,9 @@ class DefaultPortableWriter(PortableWriter):
         self._set_position(field_name, FieldType.DOUBLE)
         self._out.write_double(value)
 
-    def write_utf(self, field_name, value):
-        self._set_position(field_name, FieldType.UTF)
-        self._out.write_utf(value)
+    def write_string(self, field_name, value):
+        self._set_position(field_name, FieldType.STRING)
+        self._out.write_string(value)
 
     def write_byte_array(self, field_name, values):
         self._set_position(field_name, FieldType.BYTE_ARRAY)
@@ -92,9 +93,9 @@ class DefaultPortableWriter(PortableWriter):
         self._set_position(field_name, FieldType.DOUBLE_ARRAY)
         self._out.write_double_array(values)
 
-    def write_utf_array(self, field_name, values):
-        self._set_position(field_name, FieldType.UTF_ARRAY)
-        self._out.write_utf_array(values)
+    def write_string_array(self, field_name, values):
+        self._set_position(field_name, FieldType.STRING_ARRAY)
+        self._out.write_string_array(values)
 
     def write_portable(self, field_name, portable):
         fd = self._set_position(field_name, FieldType.PORTABLE)
@@ -117,7 +118,7 @@ class DefaultPortableWriter(PortableWriter):
         if _len > 0:
             _offset = self._out.position()
             self._out.write_zero_bytes(_len * 4)
-            for i in xrange(0, _len):
+            for i in range(0, _len):
                 portable = values[i]
                 _check_portable_attributes(fd, portable)
                 _pos_val = self._out.position()
@@ -139,19 +140,29 @@ class DefaultPortableWriter(PortableWriter):
         self._raw = True
         return self._out
 
+    def write_utf(self, field_name, value):
+        self.write_string(field_name, value)
+
+    def write_utf_array(self, field_name, values):
+        self.write_string_array(field_name, values)
+
     # internal
     def _set_position(self, field_name, field_type):
         if self._raw:
-            raise HazelcastSerializationError("Cannot write Portable fields after get_raw_data_output() is called!")
+            raise HazelcastSerializationError(
+                "Cannot write Portable fields after get_raw_data_output() is called!"
+            )
         fd = self._class_def.get_field(field_name)
         if fd is None:
-            raise HazelcastSerializationError("Invalid field name:'{}' for ClassDefinition(id:{} , version:{} )"
-                                              .format(field_name, self._class_def.class_id, self._class_def.version))
+            raise HazelcastSerializationError(
+                "Invalid field name:'%s' for ClassDefinition(id:%s , version:%s )"
+                % (field_name, self._class_def.class_id, self._class_def.version)
+            )
         if field_name not in self._writen_fields:
             self._write_field_def(fd.index, field_name, field_type)
             self._writen_fields.add(field_name)
         else:
-            raise HazelcastSerializationError("Field '{}' has already been written!".format(field_name))
+            raise HazelcastSerializationError("Field '%s' has already been written!" % field_name)
         return fd
 
     def _write_field_def(self, index, field_name, field_type):
@@ -159,7 +170,7 @@ class DefaultPortableWriter(PortableWriter):
         position = self._offset + index * INT_SIZE_IN_BYTES
         self._out.write_int(pos_val, position)
         self._out.write_short(len(field_name))
-        self._out.write_from(field_name)
+        self._out.write_from(field_name.encode("utf-8"))
         self._out.write_byte(field_type)
 
     def end(self):
@@ -170,17 +181,23 @@ class DefaultPortableWriter(PortableWriter):
 
 def _check_portable_attributes(field_def, portable):
     if field_def.factory_id != portable.get_factory_id():
-        raise HazelcastSerializationError("Wrong Portable type! Generic portable types are not supported! "
-                                          "Expected factory-id: {}, Actual factory-id: {}"
-                                          .format(field_def.factory_id, portable.get_factory_id()))
+        raise HazelcastSerializationError(
+            "Wrong Portable type! Generic portable types are not supported! "
+            "Expected factory-id: %s, Actual factory-id: %s"
+            % (field_def.factory_id, portable.get_factory_id())
+        )
     if field_def.class_id != portable.get_class_id():
-        raise HazelcastSerializationError("Wrong Portable type! Generic portable types are not supported! "
-                                          "Expected class-id: {}, Actual class-id: {}"
-                                          .format(field_def.class_id, portable.get_class_id()))
+        raise HazelcastSerializationError(
+            "Wrong Portable type! Generic portable types are not supported! "
+            "Expected class-id: %s, Actual class-id: %s"
+            % (field_def.class_id, portable.get_class_id())
+        )
 
 
 class ClassDefinitionWriter(PortableWriter):
-    def __init__(self, portable_context, factory_id=None, class_id=None, version=None, class_def_builder=None):
+    def __init__(
+        self, portable_context, factory_id=None, class_id=None, version=None, class_def_builder=None
+    ):
         self.portable_context = portable_context
         if class_def_builder is None:
             self._builder = ClassDefinitionBuilder(factory_id, class_id, version)
@@ -211,8 +228,8 @@ class ClassDefinitionWriter(PortableWriter):
     def write_double(self, field_name, value):
         self._builder.add_double_field(field_name)
 
-    def write_utf(self, field_name, value):
-        self._builder.add_utf_field(field_name)
+    def write_string(self, field_name, value):
+        self._builder.add_string_field(field_name)
 
     def write_byte_array(self, field_name, values):
         self._builder.add_byte_array_field(field_name)
@@ -238,12 +255,14 @@ class ClassDefinitionWriter(PortableWriter):
     def write_double_array(self, field_name, values):
         self._builder.add_double_array_field(field_name)
 
-    def write_utf_array(self, field_name, values):
-        self._builder.add_utf_array_field(field_name)
+    def write_string_array(self, field_name, values):
+        self._builder.add_string_array_field(field_name)
 
     def write_portable(self, field_name, portable):
         if portable is None:
-            raise HazelcastSerializationError("Cannot write None portable without explicitly registering class definition!")
+            raise HazelcastSerializationError(
+                "Cannot write None portable without explicitly registering class definition!"
+            )
         version = util.get_portable_version(portable, self.portable_context.portable_version)
 
         factory_id = portable.get_factory_id()
@@ -254,27 +273,40 @@ class ClassDefinitionWriter(PortableWriter):
 
     def write_portable_array(self, field_name, values):
         if values is None or len(values) == 0:
-            raise HazelcastSerializationError("Cannot write None portable without explicitly registering class definition!")
+            raise HazelcastSerializationError(
+                "Cannot write None portable without explicitly registering class definition!"
+            )
         portable = values[0]
         _class_id = portable.get_class_id()
-        for i in xrange(1, len(values)):
+        for i in range(1, len(values)):
             if values[i].get_class_id() != _class_id:
                 raise ValueError("Detected different class-ids in portable array!")
         version = util.get_portable_version(portable, self.portable_context.portable_version)
-        nested_builder = ClassDefinitionBuilder(portable.get_factory_id(), portable.get_class_id(), version)
+        nested_builder = ClassDefinitionBuilder(
+            portable.get_factory_id(), portable.get_class_id(), version
+        )
         nested_class_def = self._create_nested_class_def(portable, nested_builder)
         self._builder.add_portable_array_field(field_name, nested_class_def)
 
     def write_null_portable(self, field_name, factory_id, class_id):
-        nested_class_def = self.portable_context.lookup_class_definition(factory_id, class_id,
-                                                                         self.portable_context.portable_version)
+        nested_class_def = self.portable_context.lookup_class_definition(
+            factory_id, class_id, self.portable_context.portable_version
+        )
         if nested_class_def is None:
-            raise HazelcastSerializationError("Cannot write None portable without explicitly registering class definition!")
+            raise HazelcastSerializationError(
+                "Cannot write None portable without explicitly registering class definition!"
+            )
         self._builder.add_portable_field(field_name, nested_class_def)
 
     def get_raw_data_output(self):
         return EmptyObjectDataOutput()
 
+    def write_utf(self, field_name, value):
+        self.write_string(field_name, value)
+
+    def write_utf_array(self, field_name, values):
+        self.write_string_array(field_name, values)
+
     def _create_nested_class_def(self, portable, nested_builder):
         _writer = ClassDefinitionWriter(self.portable_context, class_def_builder=nested_builder)
         portable.write_portable(_writer)
